/***********************************************************
constants
***********************************************************/

// reference to the interface defined in cmguiIGrayscaleFilter.idl
const cmguiIGrayscaleFilter = Components.interfaces.cmguiIGrayscaleFilter;

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;

// references to other required interfaces
// const nsIDOMWindow = Components.interfaces.nsIDOMWindow;

// UUID uniquely identifying our component
// You can get from: http://kruithof.xs4all.nl/uuid/uuidgen here

const CLASS_ID = Components.ID("{03684D38-B045-11DB-A232-603656D89593}");

// description
const CLASS_NAME = "Cmgui gray scale filter XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@bioeng.auckland.ac.nz/grayscalefilter;1";

/***********************************************************
class definition
***********************************************************/

//class constructor
function GrayscaleFilter() {
};


// class definition
GrayscaleFilter.prototype = {

  // members
	parent: null,
	commandData: null,
	redWeight: 0.299,
	greenWeight: 0.587,
	blueWeight: 0.114,
	inputFieldName: null,
	outputFieldName: null,
	inputField: null,
	outputField: null,
	callback: null,
	resampledInputFieldName: null,
	previewOutputFieldName: null,
	resampledInputField: null,
	previewOutputField: null,
	previewResolution: 128,
	gElementName: null,
	sceneViewer: null,

// define the function methods we want to expose in our interface

  init: function(parentWindow, commandData, inputFieldName, outputFieldName, callback) {

		this.parent = parentWindow;

		//		parentWindow.alert("Calling bob from grayscale filter xpcom: " + parentWindow.bob);
		//		parentWindow.bob = {bobtext{stringtext:"Goodbye bob"}};
		this.commandData = commandData;
    this.callback = callback;       // function to apply once field has been filtered

		this.inputFieldName = inputFieldName;
		this.outputFieldName = outputFieldName;

		// find the input field, note it is assumed to exist
	  this.inputField = commandData.findFieldByName(inputFieldName);

		// set some defaults which will be overwritten if the field exists
		// use the CIE values from Charles Poynton http://www.poynton.com/ColorFAQ.html
		this.redWeight = 0.2125;
		this.greenWeight = 0.7514;
		this.blueWeight = .0721;

		// check if output field exists
		if( commandData.isFieldDefined(outputFieldName) ){

			// field exists so fetch it
			this.outputField = commandData.findFieldByName(outputFieldName);

			// create some objects which will be populated by the current field
			// properties if the field is the right type
			
			var returnInputField = new Object();
			var weights = {};
			var count = new Object();

			try {

				this.outputField.getTypeSumComponents(returnInputField,weights,count);
				
				weights = weights.value;  // weight array has now been fetched			 
				// should round these to 3 dp as there may have been small rounding errors
				// because of casting from float to double
				this.redWeight  = Math.round(1000*weights[0])/1000;  
				this.greenWeight = Math.round(1000*weights[1])/1000;
				this.blueWeight = Math.round(1000*weights[2])/1000;

			} catch(ex) {

				this.parent.alert("Could not fetch parameters, probably because " + 
													"the output field exists but is of the wrong type " + ex);				

				weights = new Array(3);
				weights[0] = this.redWeight;
				weights[1] = this.greenWeight;
				weights[2] = this.blueWeight;
				count = 3;

				this.outputField.setTypeSumComponents(this.inputField,weights,count);

			}
			
		} else {

			// field does not exist so create it.

			this.outputField = commandData.createField(outputFieldName);
			
			weights = new Array(3);
			weights[0] = this.redWeight;
			weights[1] = this.greenWeight;
			weights[2] = this.blueWeight;
			count = 3;
			
			this.outputField.setTypeSumComponents(this.inputField,weights,count);
			
		}	 
		
  },


	setup_preview_field: function() {


		// note we assume weights now exist
		
		// first resample the input field
		// will eventually use createField and setType to do this
		this.resampledInputFieldName = "resampled_" + this.inputFieldName;
		this.commandData.executeCommand("gfx def field " + this.resampledInputFieldName +
 		  " image_resample field " + this.inputFieldName + " sizes " +
			this.previewResolution + " " + this.previewResolution + ";");
		this.resampledInputField = this.commandData.findFieldByName(this.resampledInputFieldName);
		
 		// Set up previewOutputField
		this.previewOutputFieldName = "preview_" + this.outputFieldName;
		this.previewOutputField = this.commandData.createField(this.previewOutputFieldName);
			
		var weights = new Array(3);
		weights[0] = this.redWeight;
		weights[1] = this.greenWeight;
		weights[2] = this.blueWeight;
		var count = 3;

		// set the type with the current options
		this.previewOutputField.setTypeSumComponents(this.resampledInputField,weights,count);

	},

	
  apply_filter: function() {

		var weights = new Array(3);
		weights[0] = this.redWeight;
		weights[1] = this.greenWeight;
		weights[2] = this.blueWeight;

		// This is the line that seems to cause problems
		this.outputField.setTypeSumComponents(this.inputField,weights,3);

    // execute desired behaviour after filter application by using callback function
    // currently we are using a 0 argument callback function.  This may need to be replaced
    // with a 1 or 2 argument callback function to be more generalised and more useful.
    this.callback.run();


  },


	resample_preview: function() {

		this.commandData.executeCommand("gfx def field " + this.resampledInputFieldName +
			" image_resample field " + this.inputFieldName + " sizes " +
			this.previewResolution + " " + this.previewResolution + ";");

	},

	// should this return the name of the material?  change this to update_preview_field
  preview_filter: function() {

		var weights = new Array(3);
		weights[0] = this.redWeight;
		weights[1] = this.greenWeight;
		weights[2] = this.blueWeight;

		// update the preview output field options
		this.previewOutputField.setTypeSumComponents(this.resampledInputField,weights,3);
		
		// need to evaluate the image
		// what goes here?  
		// should be similar to refilter plus some of display_slice logic
		// will need to have different textures for different filter dialogs 
		// so that multiple previews can be used at once.


  },


	create_dialog: function() {

		var parent = this.parent;
		var params={grayscaleFilter:this};

		this.setup_preview_field();

		try {

			dialogReference = parent.openDialog("chrome://zinc/content/filters/grayscaleFilterDialog.xul",
																							"gray-scale-filter-dialog",
																							"chrome",params);

			// store reference to dialog and associated field name
			parent.grayscaleFilterDialogReference = dialogReference;
			parent.grayscaleFilterDialogReferenceOutputField = this.outputFieldName;

		}catch(ex) {
			parent.alert("Error creating dialog to edit grayscale filter: " + ex);
		}

	},


  open_dialog: function(gElementName, sceneViewer) {

		this.gElementName = gElementName;
		this.sceneViewer = sceneViewer;

		// parent window
		var parent = this.parent;

		var dialogReference = parent.grayscaleFilterDialogReference;
		
		// if no reference to an open dialog already exists create it
		if (dialogReference == null || dialogReference.closed) {
			
			// no dialog so create one
			this.create_dialog();
			
		} else {
			
			// dialog exists, check if it is relevant to this field.				
			var oldOutputFieldName = parent.grayscaleFilterDialogReferenceOutputField;
			
			if (oldOutputFieldName == this.outputFieldName) {
				
				// correct dialog so bring it to the front
				dialogReference.focus();
				
			} else {
				
				// not correct so close dialog and open a new one
				dialogReference.close();
				this.create_dialog();
				
			}
		}
			
  },



    
  // the QueryInterface function throws an exception
  // if the cmguiIGrayscaleFilter interface is not implemented
  // otherwise it returns a reference to the GrayscaleFilter object
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(cmguiIGrayscaleFilter) &&    
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. Eg:

myGrayscaleFilter = Components.classes["@bioeng.auckland.ac.nz/helloworld;1"].
                          createInstance(Components.interfaces.cmguiIGrayscaleFilter);

***********************************************************/
var GrayscaleFilterFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new GrayscaleFilter()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var GrayscaleFilterModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, 
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return GrayscaleFilterFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return GrayscaleFilterModule; }
